package xyz.veronie.bgg.ui.parts;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.result.BggApi;
import xyz.veronie.bgg.result.ResultConfig;
import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.result.ThingMetaData;
import xyz.veronie.bgg.result.ThingProvider;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.ResultAction;
import xyz.veronie.bgg.types.SourceFilter;
import xyz.veronie.bgg.types.Subtype;
import xyz.veronie.bgg.ui.filters.AgeSourceFilter;
import xyz.veronie.bgg.ui.filters.BggUserSourceFilter;
import xyz.veronie.bgg.ui.filters.FamilySourceFilter;
import xyz.veronie.bgg.ui.filters.GeeklistSourceFilter;
import xyz.veronie.bgg.ui.filters.RankSourceFilter;
import xyz.veronie.bgg.ui.filters.SearchSourceFilter;
import xyz.veronie.bgg.ui.filters.YearSourceFilter;



/// Has controls to download a list of thing IDs (+some meta info) from BGG.
/// There are different ways to configure which IDs to retrieve.
public class LoadFromBggPart {

	@Inject
	private ResultConfigManager configManager;
	
	@Inject private IEventBroker eventBroker;
	
	// inject all source filter composites
	@Inject private BggUserSourceFilter bggUserSourceFilter;
	@Inject private GeeklistSourceFilter geeklistSourceFilter;
	@Inject private FamilySourceFilter familySourceFilter;
	@Inject private AgeSourceFilter ageSourceFilter;
	@Inject private RankSourceFilter rankSourceFilter;
	@Inject private YearSourceFilter yearSourceFilter;
	@Inject private SearchSourceFilter searchSourceFilter;
	@Inject private BggApi bggApi;
	
	@PostConstruct
	public void createControls(Composite parent) {
		Composite main = new Composite(parent, SWT.FILL);
		main.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		
		GridLayout layout = new GridLayout(5, false);
		main.setLayout(layout);
		
		
//		// contains configuration options for download of the bgg thing ID list
//		Group dlConfigGroup = new Group(main, SWT.SHADOW_ETCHED_IN);
//		GridData configGrid = new GridData(SWT.FILL, SWT.FILL, true, true);
//		dlConfigGroup.setText("Configure filters...");
//		dlConfigGroup.setLayoutData(configGrid);
//		GridLayout layoutG = new GridLayout(2, false);
//		dlConfigGroup.setLayout(layoutG);
		
		Label lblSource = new Label(main, SWT.LEFT);
		lblSource.setText("Source: ");
		lblSource.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		
		// choose the bgg data source
		ComboViewer cbSource = new ComboViewer(main, SWT.READ_ONLY);
		cbSource.getCombo().setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		cbSource.setContentProvider(ArrayContentProvider.getInstance());
		List<SourceFilter> sources = new ArrayList<SourceFilter>();
		for(SourceFilter sf : SourceFilter.values()) {
			sources.add(sf);
		}
		cbSource.setInput(sources);
		// TODO: implement all the sources
		cbSource.setSelection(new StructuredSelection(configManager.getResultConfig().source));
		// listener is configured further below

		makeEmptySpace(main, 1);
		
		// choose the thing sub-type
		Label lblSubtype = new Label(main, SWT.LEFT);
		lblSubtype.setText("Subtypes: ");
		lblSubtype.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
		
		ComboViewer cbSubtypes = new ComboViewer(main, SWT.READ_ONLY);
		cbSubtypes.getCombo().setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		cbSubtypes.setContentProvider(ArrayContentProvider.getInstance());
		List<Subtype> subtypes = new ArrayList<Subtype>();
		for (Subtype st : Subtype.values()) { 
			subtypes.add(st);
		}
		cbSubtypes.setInput(subtypes);
		cbSubtypes.setSelection(new StructuredSelection(configManager.getResultConfig().subtype));
		cbSubtypes.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				eventBroker.send(EventConstants.TOPIC_SUBTYPE_CHANGED, selection.getFirstElement());
			}
		});
		
		

		// next row:
		
        // area to display filter controls in
		Group gFilters = new Group(main, SWT.SHADOW_ETCHED_IN);
        gFilters.setText("");
        GridData gdf = new GridData(SWT.FILL, SWT.FILL, false, false);
        gdf.horizontalSpan = 5;
        gFilters.setLayoutData(gdf);
        
		showFilter(gFilters, (IStructuredSelection)cbSource.getSelection()); 		// set content to initial selection
		cbSource.addSelectionChangedListener(new ISelectionChangedListener() {      // later, its content will be set via the dropdown
		    @Override
		    public void selectionChanged(SelectionChangedEvent event) {
		        IStructuredSelection selection = (IStructuredSelection) event.getSelection();
	        	// ASK: this is strange to me. Why is filterComposite known INSIDE a free function?
		        showFilter(gFilters, selection);
		        eventBroker.send(EventConstants.TOPIC_SOURCE_CHANGED, selection.getFirstElement());
		    }

		});
		

		// next row:
		makeEmptySpace(main, 5);

		
		// next row
		
		// choose action on result list
		Button btDownload = new Button(main, SWT.PUSH);
		GridData gd = new GridData(SWT.LEFT, SWT.FILL, false, false);
		gd.horizontalSpan = 2;
		btDownload.setLayoutData(gd);
		btDownload.setText("Fetch");
		btDownload.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
		        ResultConfig resultConfig = configManager.getResultConfig();
		        
		        IStructuredSelection selection = (IStructuredSelection) cbSource.getSelection();
		        if(selection.size() == 0) return;

		        if(selection.getFirstElement() == SourceFilter.BGG_USER) {
		        	String user = resultConfig.user;
		        	if(user == null || user.isEmpty()) {
		        		MessageDialog.openError(main.getShell(), "", "Please enter a user name.");
		        		return;
		        	} else {
				        eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching " + selection.getFirstElement().toString() + " '" + user + "'...");
				        try {
				        	ArrayList<ThingMetaData> thingMetas = bggApi.getThingsForUser(user);
				        	useThingsOnResult(thingMetas);
				        }
				        catch(IllegalArgumentException ex) {
				        	MessageDialog.openError(main.getShell(), "", ex.getMessage());
				        }
		        	}
		        	
		        } else if(selection.getFirstElement() == SourceFilter.GEEKLIST) {
		        	Integer geeklistId = resultConfig.geeklistId;
		        	if(geeklistId == null) {
		        		MessageDialog.openError(main.getShell(), "", "Please enter a geeklist id.");
		        		return;
		        	} else {
		        		eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching for geeklist id '" + geeklistId + "'");
		        		try {
		        			ArrayList<ThingMetaData> thingMetas = bggApi.getThingsForGeeklist(geeklistId);
		        			useThingsOnResult(thingMetas);
		        		}
				        catch(IllegalArgumentException ex) {
				        	MessageDialog.openError(main.getShell(), "", ex.getMessage());
				        }
		        	}
		        } else if(selection.getFirstElement() == SourceFilter.FAMILY) {
		        	Integer familyId = resultConfig.familyId;
		        	if(familyId == null) {
		        		MessageDialog.openError(main.getShell(), "", "Please enter a family id.");
		        		return;
		        	} else {
		        		eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching for family id '" + familyId + "'");
		        		try {
		        			ArrayList<ThingMetaData> thingMetas = bggApi.getThingsForFamily(familyId);
		        			useThingsOnResult(thingMetas);
		        		}
				        catch(IllegalArgumentException ex) {
				        	MessageDialog.openError(main.getShell(), "", ex.getMessage());
				        }
		        	}
		        }
		    }

			private void useThingsOnResult(ArrayList<ThingMetaData> thingMetas) {
				switch(configManager.getResultConfig().action) {
				case REP:
					ThingProvider.INSTANCE.replaceThingMetas(thingMetas);
					break;
				case ADD:
					ThingProvider.INSTANCE.addThingMetas(thingMetas);
					break;
				case SUB:
					ThingProvider.INSTANCE.subtractThingMetas(thingMetas);
					break;
				case AND:
					ThingProvider.INSTANCE.intersectThingMetas(thingMetas);
					break;
				}
				eventBroker.send(EventConstants.TOPIC_RESULT_CHANGED, "");
				eventBroker.send(EventConstants.TOPIC_STATUS, "Fetched " + Integer.toString(thingMetas.size()) + " things.");
			}
		});

		
		Label lblAct = new Label(main, SWT.LEFT);
		lblAct.setText("and");
		lblAct.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, false, false));
		
		ComboViewer cbAct = new ComboViewer(main, SWT.READ_ONLY);
		GridData gdact = new GridData(SWT.FILL, SWT.FILL, false, false);
		cbSubtypes.getCombo().setLayoutData(gdact);
		cbAct.setContentProvider(ArrayContentProvider.getInstance());
		List<ResultAction> actions = new ArrayList<ResultAction>();
		for(ResultAction act : ResultAction.values()) {
			actions.add(act);
		}
		cbAct.setInput(actions);
		cbAct.setSelection(new StructuredSelection(configManager.getResultConfig().action));
		cbAct.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				eventBroker.send(EventConstants.TOPIC_ACTION_CHANGED, selection.getFirstElement());
			}
		});
		
		
		Label toResultLabel = new Label(main, SWT.LEFT);
		toResultLabel.setText("result");
		toResultLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));        
        
//		// Start retrieval when download button is pushed
//		// TODO: how do I locate icons?
//		Button btDownload = new Button(main, SWT.PUSH);
//		try {
//			Image dlIcon = new Image(getDisplay(), 
//				getClass().getResourceAsStream("C:\\Users/goya/workspace/icons/icons/material_design/png/download-button.png"));
//			btDownload.setImage(dlIcon);
//		} 
//		catch(Exception ex) {
//			btDownload.setText("Download now");
//		}

		main.pack();
		
	}
	
	
	/// show different filter controls depending on selection in cbSource ComboViewer
	public void showFilter(Composite parent, IStructuredSelection selection) {	
		SourceFilter elem = (SourceFilter) selection.getFirstElement();

		// clean up
		for(Control child : parent.getChildren()) {
			child.dispose();
		}
		
		// create a new filter area based on selection:
		switch(elem) {
		case BGG_USER:
			System.out.println("construct " + elem);
			bggUserSourceFilter.create(parent, SWT.FILL);
			break;
		case GEEKLIST:
			System.out.println("construct " + elem);
			geeklistSourceFilter.create(parent, SWT.FILL);
			break;
		case FAMILY:
			System.out.println("construct " + elem);
			familySourceFilter.create(parent, SWT.FILL);
			break;
		case RANK:
			rankSourceFilter.create(parent, SWT.FILL);
			break;
		case YEAR:
			yearSourceFilter.create(parent, SWT.FILL);
			break;
		case AGE:
			ageSourceFilter.create(parent, SWT.FILL);
			break;
		case SEARCH:
			searchSourceFilter.create(parent, SWT.FILL);
			break;
		default:
			System.out.println("construct " + elem + " (not implemented yet, try another one)");
		}
		
	}
	
	
	public static Display getDisplay() {
 	   Display display = Display.getCurrent();
 	   // may be null if outside the UI thread
	   if (display == null)
	      display = Display.getDefault();
	   return display;		
    }
	
	
	public void makeEmptySpace(Composite parent, int span) {
		Label empty = new Label(parent, SWT.LEFT);
        empty.setText("");
		GridData gde = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		if(span > 1) {
			gde.horizontalSpan = span;			
		}
        empty.setLayoutData(gde);
	}

}
